#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/stat.h>
#include <limits.h>
#include <dirent.h>
#include <ctype.h>
#include <libgen.h>

#include "dh5tool.h"
#include "log.h"

#define DEBUG_DH5TOOL 1

int DH5TOOL_global_rank;
int DH5tool_requires_multiple_paths = 0;
int DH5TOOL_PATH_length = 0;
int DH5TOOL_PATH_NEXT_length = 0;

FILE *DH5TOOL_debug_stream;
DH5TOOL_loglevel DH5TOOL_debug_level;

char *DH5TOOL_PATH_NEXT = NULL;
char *DH5TOOL_PATH;
char *DH5TOOL_ARGS;
char *DH5TOOL_NAME;


void
DH5TOOL_start(CIRCLE_handle *handle)
{
    handle->enqueue(DH5TOOL_PATH);
}

void
DH5TOOL_search(CIRCLE_handle *handle)
{
    DIR *current_dir;

    char temp[CIRCLE_MAX_STRING_LEN];
    char stat_temp[CIRCLE_MAX_STRING_LEN];

    struct dirent *current_ent;
    struct stat st;

    /* Pop an item off the queue */
    handle->dequeue(temp);

    /* Try and stat it, checking to see if it is a link */
    if(lstat(temp,&st) != EXIT_SUCCESS)
    {
        LOG(DH5TOOL_LOG_ERR, "Error: Couldn't stat \"%s\"", temp);
        return;
    }
    /* Check to see if it is a directory.  If so, put its children in the queue */
    else if(S_ISDIR(st.st_mode) && !(S_ISLNK(st.st_mode)))
    {
        current_dir = opendir(temp);

        if(!current_dir)
        {
            LOG(DH5TOOL_LOG_ERR, "Unable to open dir");
        }
        else
        {
            /* Read in each directory entry */
            while((current_ent = readdir(current_dir)) != NULL)
            {
            /* We don't care about . or .. */
            if((strncmp(current_ent->d_name,".",2)) && (strncmp(current_ent->d_name,"..",3)))
                {
                    strcpy(stat_temp,temp);
                    strcat(stat_temp,"/");
                    strcat(stat_temp,current_ent->d_name);

                    handle->enqueue(&stat_temp[0]);
                }
            }
        }
        closedir(current_dir);
    }
    else if(S_ISREG(st.st_mode)) {
        FILE *fp;
        char cmd[CIRCLE_MAX_STRING_LEN + 10];
        char out[1035];
	char *dot = strrchr(temp,'.'); /* Dot H5 file extension */
	int enable_tool = 1;
	/* Only apply the HDF5 tools on an HDF5 file... 
	 * TODO: should extend this to handle (NetCDF) .nc files as well!
	 */
	
	if (dot && (strcasecmp(dot,".h5") == 0)) {
	    if (DH5tool_requires_multiple_paths) {
		char pathNext[PATH_MAX];
		struct stat statbuf;
		if (temp[DH5TOOL_PATH_length] == '/')
		    sprintf(pathNext,"%s%s",DH5TOOL_PATH_NEXT,&temp[DH5TOOL_PATH_length]);
		else sprintf(pathNext,"%s/%s",DH5TOOL_PATH_NEXT,&temp[DH5TOOL_PATH_length]);

		if (DH5TOOL_ARGS)
		    sprintf(cmd, "%s %s %s %s", DH5TOOL_NAME, DH5TOOL_ARGS, temp, pathNext);
		else sprintf(cmd, "%s %s %s", DH5TOOL_NAME, temp, pathNext);
		if (stat(pathNext, &statbuf) != 0)
		    enable_tool = 0;
	    }
	    else if (DH5TOOL_ARGS) 
		sprintf(cmd, "%s %s %s", DH5TOOL_NAME, DH5TOOL_ARGS, temp);
	    else sprintf(cmd, "%s %s", DH5TOOL_NAME, temp);

#if DEBUG_DH5TOOL
	    puts("-----------------------");
	    printf("running: %s\n",cmd);
	    fflush(stdout);
#endif
	    if (enable_tool) {
		fp = popen(cmd, "r");
		if (fp == NULL) {
		    printf("Failed to run command\n" );
		}

		while (fgets(out, sizeof(out) - 1, fp) != NULL) {
		    printf("%s", out);
		}
		pclose(fp);
	    }
	    else printf("skipping file: %s\n", temp);
	}
    }
}

void
print_usage(char *prog)
{
    fprintf(stdout, "Usage: %s -p <path> -t <h5tool> -a <h5tool args>\n", prog);
}

int
main (int argc, char **argv)
{
    int index;
    int c;

    int args_flag = 0;
    int path_flag = 0;
    int tool_flag = 0;

    DH5TOOL_debug_stream = stdout;
    DH5TOOL_debug_level  = DH5TOOL_LOG_DBG;
     
    opterr = 0;
    while((c = getopt(argc, argv, "a:p:t:")) != -1)
    {
        switch(c)
        {
            case 'a':
                DH5TOOL_ARGS = optarg;
                args_flag = 1;
                break;
            case 'p':
		/* The h5diff tool needs two(2) paths
		 * We'll allow that possiblity with a 2nd -p argument.
		 */
		if (path_flag == 0) {
		    DH5TOOL_PATH = realpath(optarg, NULL);
		    DH5TOOL_PATH_length = strlen(DH5TOOL_PATH);		    
		}
		else {
		    DH5TOOL_PATH_NEXT = realpath(optarg, NULL);
		    DH5TOOL_PATH_NEXT_length = strlen(DH5TOOL_PATH_NEXT);
		}
                path_flag = 1;
                break;
            case 't':
		DH5TOOL_NAME = optarg;
		tool_flag = 1;
                break;
            case '?':
                if (optopt == 'p' || optopt == 't')
                    fprintf(stderr, "Error: Option -%c requires an argument.\n", optopt);
                else if (isprint (optopt))
                    fprintf(stderr, "Error: Unknown option `-%c'.\n", optopt);
                else
                    fprintf(stderr,
                        "Error: Unknown option character `\\x%x'.\n",
                        optopt);

                exit(EXIT_FAILURE);
            default:
                abort();
        }
    }

    for (index = optind; index < argc; index++)
    {
        print_usage(argv[0]);
        fprintf(stdout, "Error: Non-option argument %s\n", argv[index]);

        exit(EXIT_FAILURE);
    }

    if(path_flag == 0)
    {
        print_usage(argv[0]);
        fprintf(stdout, "Error: You must specify a starting path name.\n");

        exit(EXIT_FAILURE);
    }

    if (strstr(DH5TOOL_NAME,"h5diff")) {
	DH5tool_requires_multiple_paths = 1;
    }

    DH5TOOL_global_rank = CIRCLE_init(argc, argv, CIRCLE_DEFAULT_FLAGS);

    CIRCLE_cb_create (&DH5TOOL_start);
    CIRCLE_cb_process(&DH5TOOL_search);

    CIRCLE_begin();
    CIRCLE_finalize();

    exit(EXIT_SUCCESS);
}

/* EOF */
